#!/usr/bin/env python

#General Modules
import RPi.GPIO as GPIO
from os import system, path
import math
from time import sleep
import threading 			#to run threaded process in background
import subprocess 			#to run external scripts like perl
import pigpio				#modules for RPM sensing and driving bushless motor
from read_RPM import reader	#free sample to read RPM
import serial				#Read PWM Value

#get the current running directory
moduleLocation = path.dirname(path.realpath(__file__))
moduleLocation = moduleLocation+"/"
#print moduleLocation

#Custom Modules
#import poll
import oled
#from rpIp import get_ip_address

#GPIO assignments (BCM)
power =21 #shutdown button
center=20
up    =26
down  =13
left  =16
right =19
rpmPin = 6 #input pin to sense RPM
escPin = 12 #output to the ESC

reflectionPoints = 2 #number of reflection points on motor
pwmJitterReduction = 4 #only change pwm value when value has a deviation larger than listed

escMinSignal = 800 			#min frequency to keep ESC from beeping
motorSpeed   = escMinSignal	#full frequency after user/pwm input

# serial Variables
serialPwmPort  = None # serial.Serial()
serialPwmDevice      = "/dev/ttyAMA0"
serialPwmBaud  = 115200
serialGrblPort = None # serial.Serial()
serialGrblDevice     = "/dev/ttyUSB0"
serialGrblBaud = 115200

#gcodeFile               = None
#gcodeArray               = []
gcodeFileLocation        = "NA"
#gcodeFileSearchLocatoin = "/home/pi/Documents"
#gcodeFileSearchType     = "\'.nc|.tap\'" #ls -la /home/pi/Documents | grep -E ".nc|.tap"
serialVerbose            = False # set to True if you want to see GRBL commands live
serialSettingsMode       = False
#l_count                  = 0
#g_count                  = 0
#c_line                   = []


grblLogFile = moduleLocation+"debugLatestJob.log"

#perl helper scripts TODO: redo in python
networkPl = moduleLocation+"network.pl"
devPl = moduleLocation+"devList.pl"
grblFilePl = moduleLocation+"grblFileList.pl"


#initialize
#def initialize():
print 'initializing..'
#initialize global variables

currentMenu = "mainMenu"
mainMenu	= 	[
			["Config GRBL","grblMenu"],
			["Jog Axis","movementMenu"],
			["Spindle","spindleMenu"],
			["Job Menu","jobMenu"],
			["PC Info","pcMenu"],
			["Network","networkMenu"],
			["Power Down","powerMenu"]
			]
			
grblMenu    = 	[
			["$serialGrblDeviceText","grblDevMenu"], #ls /dev | grep tty | grep -Ei "usb|ama"
			["Connect GRBL:","NA"],
			["$enableGrblText","NA"],
			["$unlockCncText","NA"],
			["$resetCncText","NA"],
			["Back..","mainMenu"]
			]

grblDevMenu = [] #grblMenu
			
movementMenu= 	[
			["$homeText","NA"],
			["$goToZeroText","NA"],
			["$setZeroText","NA"],
			["Jog Amount:","NA"],
			["$moveIncrimentText","NA"],
			["$moveXText","NA"],
			["$moveYText","NA"],
			["$moveZText","NA"],
			["Back..","mainMenu"]
			]
			
spindleMenu	= 	[
			["Manual Motor:","NA"],
			["$manualMotorValueText","NA"],
			["Enable Motor:","NA"],
			["$enableMotorText","NA"],
			["Connect PWM:","NA"],
			["$enablePwmText","NA"],
			["GRBL PWM Value:","NA"],
			["$grblPwmValueText","NA"],
			#["RPM ","NA"],
			["$rpmText","NA"],
			["Back..","mainMenu"]
			]
			
jobMenu		=	[
			["Select Job","jobListMenu"],
			["Test Job","jobTestMenu"], 
			["Enable Motor:","NA"],
			["$enableMotorText","NA"], 
			["Run Job","jobRunningMenu"], 
			["Back..","mainMenu"]
			] 
			
jobListMenu	=	[] #jobMenu

jobTestMenu    =	[
			["$jobStatus","NA"],
			["Cancel Test","cancelJob()"],
			["Back..","jobMenu"]
			]

jobRunningMenu =	[
			["$rpmText","jobListMenu"],
			["$jobStatus","NA"],
			["Manual Motor:","NA"],
			["$manualMotorValueText","NA"],
			["Cancel Job","cancelJob()"],
			["Back..","jobMenu"]
			]

pcMenu		= 	[
			["$HostName","NA"],
			["$libraryCheck","NA"],
			["$mlmVersion","NA"],
			["Back..","mainMenu"]
			]
networkMenu	= 	[
			["Network Info","netInfoMenu"], 
			["Connect to Netowrk",""], 
			["Back..","mainMenu"]
			]
netInfoMenu	=	[
			["IP:","NA"],
			["$ip","NA"],
			["SSID:","NA"],
			["$ssid","NA"],
			["WIFI Signal:","NA"],
			["$signalLevel","NA"],
			["Back..","networkMenu"]
			]
			
netListMenu	=	[] #netInfoMenu

pwmDevMenu  = [] #spindleMenu

powerMenu      = 	[
			["Please","powerDown()"],
			["Back..","mainMenu"]
			]


#initialize variables 

menuLocation         = 0 #highlighted location on OLED
topOledText          = " " #what whsould be drawn on top half of OLED
topOledTextPrev      = " "
bottomOledText       = " "
bottomOledTextPrev   = " "
topOledhighlight     = True
topOledhighlightPrev = True


#jobCommandCount = 0 #global counter to keep track of commands sent from file
jobStatus       = "Job: 0%"
jobRunning      = False #delete and just check if thread is running!!

HostName       	= "NA" #No code to set value yet
libraryCheck   	= "NA" #No code to set value yet
mlmVersion	   	= "NA" #No code to set value yet

ip             	= "NA"
ssid           	= "NA"
signalLevel    	= "NA"

manualMotorValue     = 0
manualMotorValueText = "0%"
enableMotor          = False
enableMotorText      = "False"
enablePwm            = False
enablePwmText        = "False"
grblPwmValue         = 0
grblPwmValueText     = "0"
rpmText 		         = "0"

serialGrblDeviceText = "Select Device"
enableGrbl     = False
enableGrblText = "False"
unlockCncText  = "Unlock"
resetCncText   = "Reset"

homeText      = "Home"
goToZeroText  = "Go to Zero"
setZeroText   = "Zero All Axis"
moveIncriment     = 1.0
moveIncrimentText = str(moveIncriment)+"mm"
moveXText     = "- Move X +"
moveYText     = "- Move Y +"
moveZText     = "- Move Z +"

#gcode for basic machine movement
gcodeHome = "$H" #not gocode but easier to group variables
gcodeGoToZero = "G90G0X0Y0Z0"
gcodeSetZero = "G10L20P1X0Y0Z0"
gcodeUnlock = "$X" #not gocode but easier to group variables
gcodeReset = "S2000"
gcodeUnitMm  = "G21"	#Not yet used!!
gcodeUnitInch = "G20"	#Not yet used!!
gcodeJogX     = "G91G0X" #need to append movement amount
gcodeJogY     = "G91G0Y" #need to append movement amount
gcodeJogZ     = "G91G0Z" #need to append movement amount

#gcode 1.1
gcodTest = "$C" # not used yet


#determin how long a button (only applies to left and right buttons) should
##be pressed before clearing previous button press
button = "NA" 				#currently pressed button
buttonPrev = "NA"			#prevously presssed button
buttonPressedCount = 0 		#running count/how long button is pressed
buttonPressedMaxCount = 500#max count before clearing prevous button press

#connect to pigpio
pi =  pigpio.pi()
tach = reader(pi, rpmPin, reflectionPoints) #2 reflection points per revolution
#initialize esc
pi.set_servo_pulsewidth(escPin, escMinSignal)
'''
#calibrate ESC
pi.set_servo_pulsewidth(escPin, 2500) #max throttle
sleep(2)
pi.set_servo_pulsewidth(escPin, (escMinSignal + 50)) #min throttle 
sleep(2)
'''


#set event listiner for gpio pins
GPIO.setup(power, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(up, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(down, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(left, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(right, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(center, GPIO.IN, pull_up_down=GPIO.PUD_UP)

#Global grbl counters
#def clearGrblCounters():
#	global jobStatus
#	global l_count
#	global g_count
#	global c_line
#
#	jobStatus = "Job: 0%"
#	l_count = 0
#	g_count = 0
#	c_line = []	
	
	
def cancelJob():
	global jobRunning
	global jobStatus
	
	jobRunning = False
	jobStatus = "Job: 0%"
	
	
#check if gcode file exists to start the job
def startJobCheck():
	global gcodeFileLocation
	global enableGrbl
	global jobStatus
	
	jobCheckSuccess = True
	
	#check if grbl file exists	
	try:
		#try to open grbl file
		open(gcodeFileLocation)
	except:
		jobCheckSuccess = False
		print "File Does Not exist:", gcodeFileLocation
		
	#check if connected to grbl
	if(not enableGrbl):
		jobCheckSuccess = False
		print "Not connected to GRBL"
	

	if(jobCheckSuccess):
		jobStatus = "Job: 0%"
		
	return jobCheckSuccess
	
	
#subroutine open PWM serial port
def openSerialPwmPort():
	global serialPwmPort
	global serialPwmDevice
	global serialPwmBaud
	
	successfulTry = True
	
	#useing a try/except block to not crash program
	try:
		serialPwmPort = serial.Serial(serialPwmDevice, baudrate=serialPwmBaud)
		print 'Opening PWM serial port'
	except:
		print 'Could not open port'
		successfulTry = False

	return successfulTry
		
#subroutine open Grbl serial port
def openSerialGrblPort():
	global serialGrblPort
	global serialGrblDevice
	global serialGrblBaud
	
	successfulTry = True
	
	#useing a try/except block to not crash program
	try:
		serialGrblPort = serial.Serial(serialGrblDevice, baudrate=serialGrblBaud)
		sleep(2)
		serialGrblPort.write("\r\n\r\n") # Wake up grbl
		
		serialGrblPort.flushInput() # flush startup text in serial input
		print 'Opening Grbl serial port'
		
	except:
		print 'Could not open port'
		successfulTry = False
		
	return successfulTry
	
#subroutine close PWM serial port	
def closeSerialPwmPort():
	global serialPwmPort
	
	successfulTry = True
	
	#useing a try/except block to not crash program
	try:
		serialPwmPort.close()
		print 'Closing PWM serial port'
	except:
		print 'Could not close port'
		successfulTry = False
		
	return successfulTry
		
#subroutine to close Grbl serial port	
def closeSerialGrblPort():
	global serialGrblPort
	
	successfulTry = True
	
	#useing a try/except block to not crash program
	try:
		serialGrblPort.close()
		print 'Closing Grbl serial port'
	except:
		print 'Could not close port'
		successfulTry = False
		
	return successfulTry


#subroutine to power down pi
def powerDown():
	global oled
	
	#show shutdown message on OLED
	print 'power pushed!'
	oled.draw_rectangle_full_screen(0)
	oled.draw_text_top_screen("Later", 1)
	oled.draw_text_bottom_screen("Gator . . . ", 1)
	oled.draw_image_buffer()
	sleep(1)
	#system shutdown command
	system("sudo shutdown -h now")


#subroutine to get network information
def checkIp():
	#need to state global variables are being used when threading?
	global currentMenu
	global ip
	global ssid
	global signalLevel
	
	refresh = 1
	#only poll for net information when looking in the network info menu
	while(currentMenu == "netInfoMenu"):
		#print("checking network")

		#use perl script to grab network info
		networkInfo   = subprocess.check_output(networkPl).split('|')
		networkTittle = "IP"
		ip            = networkInfo[0]
		ip            = ip+' ' #some reason add blank character helps drawing of last char?
		ssid          = networkInfo[1]
		signalLevel   = networkInfo[2]
		
		#only refresh network information every
		sleep(refresh)

def runMotor():
	global pi
	global sleep
	global enableMotor
	global enablePwm
	global manualMotorValue
	global motorSpeed
	global escPin
	global escMinSignal
	
	
	SetMotorSpeed = 0
	motorIncreaseReduction = .001 #in seconds delay before next motor increase
	motorDecreaseReduction = .01  #in seconds delay before next motor decrease
	
	#stay in loop untill motor comes to a complet stop
	while(enableMotor or enablePwm or motorSpeed > escMinSignal):
		
		#check user values for motor speed
		if(enableMotor):
			#(pulsewidth 0 or 500-2500) value set by manual motor % speed *...
			SetMotorSpeed = (manualMotorValue * 16) + escMinSignal
		elif(enablePwm):
			#(pulsewidth 0 or 500-2500) value set by pwm value *... 
			SetMotorSpeed = manualMotorValue + escMinSignal
		else:
			SetMotorSpeed = escMinSignal
			
		#refresh motor speed while highter than min value
		#if(motorSpeed > escMinSignal or SetMotorSpeed > escMinSignal):

		#cap max pulse width/speed
		if(SetMotorSpeed > 2500):
			SetMotorSpeed = 2500
			
		#increase and decrease speed "slowly"
		if(SetMotorSpeed > motorSpeed):
			motorSpeed += 1
			sleep(motorIncreaseReduction)
		elif(SetMotorSpeed < motorSpeed):
			motorSpeed -= 1
			sleep(motorDecreaseReduction)
		else: #general cpu reduction
			sleep(.001)
		
		#print "running motor",motorSpeed 
		pi.set_servo_pulsewidth(escPin, motorSpeed)
	
	

def getJobList(): 		#look into making this rutine refresh while in menu!!!
	global grblFilePl
	
	grblArray = subprocess.check_output(grblFilePl).split('|')
	return grblArray
	
#def getNetList():
def getSerialDevList():	#look into making this rutine refresh while in menu!!!
	global devPl
	
	devArray = subprocess.check_output(devPl).split('|')
	return devArray

#convert a single array into a double array supported by the menu system
def oneToTwoD(oneDArray, menu):
	twoDArray = []
	#check array is not empty
	if oneDArray:
		#start converting 1D array to a 2D array
		arryCount = 0
		for arrayEntry in oneDArray:
			#print "arrayEntry: |",arrayEntry,"| menu: |",menu,"|"
			twoDArray.append([arrayEntry,menu])
			arryCount += 1
		#add last entry to cancel
		twoDArray.append(["Cancel",menu])
	else:
		print("Array is empty")
		twoDArray = [
					["Canel",menu] # should check if menu is null
					]
	return twoDArray




#subroutine gcode for ERRORs
def testGrblGcode():
	#global serialGrblDevice
	global serialVerbose
	global serialGrblPort
	
	global jobStatus
	global jobRunning
	global gcodeFileLocation
	global grblLogFile
	
	gcodeArray       = []	#cammand values
	gcodeArrayLength = 0	#total number of commands in job
	logGrblCommands  = True
	grblLog          = None
	errorsFound      = False
	
	#check log file can be oppend and written to
	try:
		grblLog = open(grblLogFile, "w")
		grblLog.write('Grbl command log\n')
	except:
		logGrblCommands  = False
		print "Could not write to:", grblLogFile
	
	#Dump file in to an array
	for fileEntry in open(gcodeFileLocation,"r"):
		fileEntry = fileEntry.strip() #strip white spaces
		#skip comments (does not take in to account comment at the right of gcode)
		if(fileEntry.startswith("#") or fileEntry.startswith("(")):
			print "skiping comment line: ", fileEntry
		else:
			gcodeArray.append(fileEntry)
			#print "|",fileEntry,"|"
			
	# set job statues to running
	jobRunning = True
	jobStatus = "Job: 0%"
	l_count = 0
	
	#get G-code lines to find out job statistics
	if(gcodeArray): #make sure to not work on an empty array
		
		#set GRBL to test MODE
		serialGrblPort.write(gcodTest + '\n')
		
		#get the size of the array
		gcodeArrayLength = len(gcodeArray)
		#print "size of array: ", gcodeArrayLength
	
		# Send settings file via simple call-response streaming method. Settings must be streamed
		# in this manner since the EEPROM accessing cycles shut-off the serial interrupt.
		#print "SETTINGS MODE: Streaming", gcodeFileLocation.name, " to ", serialGrblDevice
		for l_block in gcodeArray:
			l_count += 1 # Iterate line counter    
			# l_block = re.sub('\s|\(.*?\)','',line).upper() # Strip comments/spaces/new line and capitalize
			#if serialVerbose: print 'SND: ' + str(l_count) + ':' + l_block,
			#serialGrblPort.write(l_block + '\n') # Send g-code block to grbl
			if serialVerbose:
				print 'SND: ' + str(l_count) + ':' + l_block,
			if logGrblCommands:
				grblLog.write('SND: ' + str(l_count) + ':' + l_block)
			
			serialGrblPort.write(l_block + '\n') # Send g-code block to grbl
			##should be waiting for ok!!
			grbl_out = serialGrblPort.readline().strip() # Wait for grbl response with carriage return
			
			if serialVerbose:
				print 'REC:',grbl_out
			if logGrblCommands:
				grblLog.write(' REC:' + str(grbl_out) + '\n')
			#update job %
			jobStatus = "Job: "+str(int((float(l_count)/float(gcodeArrayLength))*100))+"%"
			
			#
			if 'error' in grbl_out:
				errorsFound      = True
			#if job is canceled stop steaming file
			if(not jobRunning):
				#close logging File
				if(logGrblCommands):
					grblLog.close()
				cancelJob()
				
				#Exit test MODE
				serialGrblPort.write(gcodTest + '\n')
				
				#exit function
				print "Test cancelled"
				return None
		
		jobRunning = False
		
		#Exit test MODE
		serialGrblPort.write(gcodTest + '\n')
		
		if(errorsFound):
			jobStatus = "Job: ERRORs "
		else:
			jobStatus = "Job: Passed "

	
#subroutine to stream single gcode string to GRBL
def streamGrbl(gcodeCommand):
	global serialGrblDevice
	global serialVerbose
	global serialSettingsMode
	global serialGrblPort
	global enableGrbl
	#global l_count
	#global g_count
	#global c_line
	
	RX_BUFFER_SIZE = 128
	
	if(not enableGrbl):
		print "Have NOT connected to Serial port!!"

	else: #send a single command
			
		l_count = 0
		# Stream g-code to grbl
		#if serialSettingsMode:
			# Send settings file via simple call-response streaming method. Settings must be streamed
			# in this manner since the EEPROM accessing cycles shut-off the serial interrupt.
		#print "SETTINGS MODE: Streaming", serialGrblDevice

		l_block = gcodeCommand.strip() # Strip all EOL characters for consistency
		if serialVerbose: print 'SND: ' + l_block,
		serialGrblPort.write(l_block + '\n') # Send g-code block to grbl
		grbl_out = serialGrblPort.readline().strip() # Wait for grbl response with carriage return
		if serialVerbose: print 'REC:',grbl_out
		'''
		else:    
			# Send g-code program via a more agressive streaming protocol that forces characters into
			# Grbl's serial read buffer to ensure Grbl has immediate access to the next g-code command
			# rather than wait for the call-response serial protocol to finish. This is done by careful
			# counting of the number of characters sent by the streamer to Grbl and tracking Grbl's 
			# responses, such that we never overflow Grbl's serial read buffer. 
			g_count = 0
			c_line = []

			l_block = gcodeCommand.strip()
			c_line.append(len(l_block)+1) # Track number of characters in grbl serial read buffer
			grbl_out = '' 
			while sum(c_line) >= RX_BUFFER_SIZE-1 | serialGrblPort.inWaiting() :
				out_temp = serialGrblPort.readline().strip() # Wait for grbl response
				if out_temp.find('ok') < 0 and out_temp.find('error') < 0 :
					print "  Debug: ",out_temp # Debug response
				else :
					grbl_out += out_temp;
					g_count += 1 # Iterate g-code counter
					grbl_out += str(g_count); # Add line finished indicator
					del c_line[0] # Delete the block character count corresponding to the last 'ok'
			if serialVerbose: print "SND: " + l_block,
			serialGrblPort.write(l_block + '\n') # Send g-code block to grbl
			if serialVerbose : print "BUF:",str(sum(c_line)),"REC: ",grbl_out
		'''

#subroutine to stream gcode File to GRBL
def streamGrblFile():
	#global serialGrblDevice
	global serialVerbose
	global serialSettingsMode
	global serialGrblPort
	#global l_count
	#global g_count
	#global c_line
	
	global jobStatus
	global jobRunning
	global gcodeFileLocation
	global grblLogFile
	
	RX_BUFFER_SIZE   = 128
	gcodeArray       = []	#cammand values
	gcodeArrayLength = 0	#total number of commands in job
	#jobCommandCount = 0	#number of commands issued to GRBL
	logGrblCommands  = True
	grblLog          = None
	
	#check log file can be oppend and written to
	try:
		grblLog = open(grblLogFile, "w")
		grblLog.write('Grbl command log\n')
	except:
		logGrblCommands  = False
		print "Could not write to:", grblLogFile
	
	#Dump file in to an array
	for fileEntry in open(gcodeFileLocation,"r"):
		fileEntry = fileEntry.strip() #strip white spaces
		#skip comments (does not take in to account comment at the right of gcode)
		if(fileEntry.startswith("#") or fileEntry.startswith("(")):
			print "skiping comment line: ", fileEntry
		else:
			gcodeArray.append(fileEntry)
			#print "|",fileEntry,"|"
			
	# set job statues to running
	jobRunning = True
	jobStatus = "Job: 0%"
	l_count = 0
	
	#get G-code lines to find out job statistics
	if(gcodeArray): #make sure to not work on an empty array
		
		#get the size of the array
		gcodeArrayLength = len(gcodeArray)
		#print "size of array: ", gcodeArrayLength
	
		if serialSettingsMode:
			# Send settings file via simple call-response streaming method. Settings must be streamed
			# in this manner since the EEPROM accessing cycles shut-off the serial interrupt.
			#print "SETTINGS MODE: Streaming", gcodeFileLocation.name, " to ", serialGrblDevice
			for l_block in gcodeArray:
				l_count += 1 # Iterate line counter    
				# l_block = re.sub('\s|\(.*?\)','',line).upper() # Strip comments/spaces/new line and capitalize
				#if serialVerbose: print 'SND: ' + str(l_count) + ':' + l_block,
				#serialGrblPort.write(l_block + '\n') # Send g-code block to grbl
				if serialVerbose:
					print 'SND: ' + str(l_count) + ':' + l_block,
				if logGrblCommands:
					grblLog.write('SND: ' + str(l_count) + ':' + l_block)
				
				serialGrblPort.write(l_block + '\n') # Send g-code block to grbl
				grbl_out = serialGrblPort.readline().strip() # Wait for grbl response with carriage return
				
				if serialVerbose:
					print 'REC:',grbl_out
				if logGrblCommands:
					grblLog.write(' REC:' + str(grbl_out) + '\n')
				#update job %
				jobStatus = "Job: "+str(int((float(l_count)/float(gcodeArrayLength))*100))+"%"
				
				#if job is canceled stop steaming file
				if(not jobRunning):
					#close logging File
					if(logGrblCommands):
						grblLog.close()
					jobRunning = False
					
					#exit function
					print "Job cancelled"
					return None
		else:    
			# Send g-code program via a more agressive streaming protocol that forces characters into
			# Grbl's serial read buffer to ensure Grbl has immediate access to the next g-code command
			# rather than wait for the call-response serial protocol to finish. This is done by careful
			# counting of the number of characters sent by the streamer to Grbl and tracking Grbl's 
			# responses, such that we never overflow Grbl's serial read buffer. 
			
			g_count = 0 #keep track of executed gcode Lines
			c_line  = [] #track number of characters in grbl serial read buffer

			for l_block in gcodeArray:
				l_count += 1 # Iterate line counter
				# l_block = re.sub('\s|\(.*?\)','',line).upper() # Strip comments/spaces/new line and capitalize
				
				c_line.append(len(l_block)+1) #append current grbl command character count to array
				grbl_buf  =	sum(c_line)       #get latest buffer count
				grbl_out  = ''                #keep 
				
				while grbl_buf >= RX_BUFFER_SIZE-1 or serialGrblPort.inWaiting():
					out_temp = serialGrblPort.readline().strip() # Wait for grbl response
					if 'ok' in out_temp:
						if(c_line): #making sure array is not empty
							g_count += 1             # Iterate g-code counter
							grbl_out += out_temp     # append "ok" to grbl_out
							grbl_out += str(g_count) # Add line finished indicator
							del c_line[0]            # Delete the block character count corresponding to the last 'ok'
							grbl_buf  =	sum(c_line)  #get latest buffer count
						else:
							if serialVerbose:
								print 'waiting for serial to free up...\n'
							if logGrblCommands:
								grblLog.write('waiting for serial to free up...\n')
							
							#should have a max wait time before fail job
					elif 'error' in out_temp:
						
						#print erorr mess
						print "  Debug: ",out_temp # Debug response
						if logGrblCommands:
							grblLog.write("  Debug: " + out_temp + '\n')
						
						#close logging File
						if(logGrblCommands):
							grblLog.close()
						jobRunning = False
						jobStatus = "Job: ERROR "
						
						#exit function
						return None
					else: #not ok or error
						print "  Debug: ",out_temp # Debug response
						if logGrblCommands:
							grblLog.write("  Debug: " + out_temp + '\n')


				
				# Send g-code block to grbl
				if serialVerbose:
					print "SND: " + str(l_count) + " : " + l_block,
				if logGrblCommands:
					grblLog.write("SND: " + str(l_count) + " : " + l_block)
				serialGrblPort.write(l_block + '\n')
				
				# print out buffer and job status
				if serialVerbose :
					print "BUF:",str(grbl_buf),"REC:",grbl_out
				if logGrblCommands:
					grblLog.write(" BUF:" + str(grbl_buf) + " REC:" + grbl_out + '\n')
				jobStatus = "Job: "+str(int((float(g_count)/float(gcodeArrayLength))*100))+"%"
				#print "g_count:",g_count,"gcodeArrayLength:",gcodeArrayLength,"jobStatus:",jobStatus
				
				#if job is canceled stop steaming file
				if(not jobRunning):
					#close logging File
					if(logGrblCommands):
						grblLog.close()
					jobRunning = False
					
					#exit function
					print "Job cancelled"
					return None
	
	#close logging File
	if(logGrblCommands):
		grblLog.close()
		
    #set job to false when exiting sub rutine
	jobRunning = False
	

#subroutine to print top and bottom text on OLED
def printScreen(highlight,text1, text2):
	global oled
	
	oled.draw_rectangle_full_screen(False)				#clear whole screen
	oled.draw_rectangle_top_screen(highlight)			#set top half of screen to highligt value
	oled.draw_rectangle_bottom_screen(not(highlight))	#make sure bottom half of screen is opposite of top half
	oled.draw_text_top_screen(text1, not(highlight))	#write text to top half of screen
	oled.draw_text_bottom_screen(text2, highlight)		#write text to bottom half of screen
	oled.draw_image_buffer()
	
#buttonThreadAlive = False
#buttonThread = threading.Thread()

#initialize OLED
menuLocation = 0
topOledhighlight = True
topOledText = eval(currentMenu)[menuLocation][0]
bottomOledText = eval(currentMenu)[menuLocation+1][0]
printScreen(topOledhighlight, topOledText, bottomOledText)

'''
#Read in PI GPIO pins
def getButton():
	global power
	global center
	global up
	global down
	global left
	global right
	
	button = "NA"
	if (not GPIO.input(power)):
		button = "power"
	elif (not GPIO.input(center)):
		button = "center"
	elif (not GPIO.input(up)):
		button = "up"
	elif (not GPIO.input(down)):
		button = "down"
	elif (not GPIO.input(left)):
		button = "left"
	elif (not GPIO.input(right)):
		button = "right"
	
	return button
'''
	
while True:

	#initialize threads
	networkThread = threading.Thread(name='network_service', target=checkIp)
	grblTestThread = threading.Thread(name='grbl_test_service', target=testGrblGcode)
	grblThread = threading.Thread(name='grbl_stream_service', target=streamGrblFile)
	runMotorThread = threading.Thread(name='motor_service', target=runMotor)
	
	#Read in PI GPIO pins
	#button = getButton()
	button = "NA"
	if (not GPIO.input(power)):
		button = "power"
	elif (not GPIO.input(center)):
		button = "center"
	elif (not GPIO.input(up)):
		button = "up"
	elif (not GPIO.input(down)):
		button = "down"
	elif (not GPIO.input(left)):
		button = "left"
	elif (not GPIO.input(right)):
		button = "right"
		
	#only take in new button press/change in button value	
	if(button != buttonPrev):
		#Cheap debounce
		#sleep(.1)
		evalCurrentItem    = eval(currentMenu)[menuLocation][0] #get the left side text from slected location
		evalCurrentModItem = evalCurrentItem[1:]
		evalCurrentMneu    = eval(currentMenu)[menuLocation][1] #get the right side text from slected location
			
		
		#check what to do with button press
		if(button == "power"):
			powerDown()
			
		elif(button == "center"):
			#print button
			
			
			#if subrutine selected, run it
			if('(' in evalCurrentMneu):
				eval(evalCurrentMneu)

			#if new menu selected 
			elif(evalCurrentMneu != "NA"):
				
				#if its a dynamic menu first get values
				if(evalCurrentMneu == "jobListMenu"):
					#get the current list of serial devices
					jobList = getJobList()
					#convert list in to working menu
					jobListMenu = oneToTwoD(jobList, "jobMenu")
				elif(evalCurrentMneu == "grblDevMenu"):
					#get the current list of serial devices
					grblDevList = getSerialDevList()
					#convert list in to working menu
					grblDevMenu = oneToTwoD(grblDevList, "grblMenu")
				
				
				#if a menu is a selection menu, save approprate entry to a variable
				if(not evalCurrentItem == "Cancel"): #Dont save selection if cancel is selected
					if(currentMenu == "jobListMenu" ):
						gcodeFileLocation = "/home/pi/Documents/"+evalCurrentItem
						print "seting gcodeFileLocation:",gcodeFileLocation
					elif(currentMenu == "grblDevMenu" ):
						serialGrblDevice = "/dev/"+evalCurrentItem
						print "seting GRBL device to",serialGrblDevice
				
				#set back menu if pre reqs for running job not met
				if(evalCurrentItem == "jobTestMenu" and not jobRunning):
					#check all pre reqs are met
					if(not startJobCheck()):
						#dont set new menu if job pre rec's not met
						evalCurrentMneu = currentMenu
				elif(evalCurrentItem == "jobRunningMenu" and not jobRunning):
					#check all pre reqs are met
					if(not startJobCheck()):
						#dont set new menu if job pre reqs not met
						evalCurrentMneu = currentMenu
					
					
				#make the selected menu be the current menu
				currentMenu = evalCurrentMneu
				print (currentMenu)
				
				#reset menu location and highlight location
				menuLocation = 0
				topOledhighlight = True
				
				#Start networking info thread while current menu is netInfoMenu
				if(currentMenu == "netInfoMenu"): #should make sure thread is not alive
					#networkThread = threading.Thread(name='network_service', target=checkIp)
					networkThread.start()
				elif(currentMenu == "jobTestMenu"):
					if(not jobRunning): #should just check if thread is running instead?
						#Start grbl file streaming thread
						#grblThread = threading.Thread(name='grbl_stream_service', target=streamGrblFile)
						grblTestThread.start()
					else:
						print "Job already running"	
				elif(currentMenu == "jobRunningMenu"):
					if(not jobRunning): #should just check if thread is running instead?
						#Start grbl file streaming thread
						#grblThread = threading.Thread(name='grbl_stream_service', target=streamGrblFile)
						grblThread.start()
					else:
						print "Job already running"	
			
			#modifiable variable activated by center press
			elif(evalCurrentModItem == "unlockCncText"):
				streamGrbl(gcodeUnlock)
			elif(evalCurrentModItem == "resetCncText"):
				streamGrbl(gcodeReset)
			elif(evalCurrentModItem == "homeText"):
				streamGrbl(gcodeHome)
			elif(evalCurrentModItem == "goToZeroText"):
				streamGrbl(gcodeGoToZero)
			elif(evalCurrentModItem == "setZeroText"):
				streamGrbl(gcodeSetZero)
		
		elif(button == "up"):
			#print button
			
			#Dont scroll past the top of the menu
			if(menuLocation > 0):
				menuLocation = menuLocation - 1
				
				#if your scrolling up and bottom of oled is highlighted
				if(not(topOledhighlight)):
					#update highlight location
					topOledhighlight = True
			#print(menuLocation)
			
		elif(button == "down"):
			#print button
			
			#Dont go bellow bottom of menu
			if(menuLocation < len(eval(currentMenu))-1):
				menuLocation = menuLocation + 1
				
				#if your scrolling down and top of oled is highlighted
				if(topOledhighlight):
					#update highlight location
					topOledhighlight = False
			#print(menuLocation)
		
		# Left and right buttons used to modify variables	
		elif(button == "left"):
			#print button
			
			#check if variable is modifiable and modify appropriately
			if(evalCurrentModItem == "manualMotorValueText"):
				if(manualMotorValue > 0):
					manualMotorValue  = manualMotorValue -1
					manualMotorValueText = str(manualMotorValue)+"%"
			elif(evalCurrentModItem == "enableMotorText"):
				enableMotor = False
				enableMotorText = str(enableMotor)
			elif(evalCurrentModItem == "enablePwmText"):
				if(enablePwm): #only try to enable if its currently Not enabled
					if(closeSerialPwmPort()): #only change the values if port was successfully opend
						enablePwm = False
						enablePwmText = str(enablePwm)
				#clear pwm value
				grblPwmValue         = 0
				grblPwmValueText     = "0"
			elif(evalCurrentModItem == "enableGrblText"):
				if(enableGrbl):
					if(closeSerialGrblPort()):
						enableGrbl = False
						enableGrblText = str(enableGrbl)
			elif(evalCurrentModItem == "moveIncrimentText"):
				if(moveIncriment > .25):
					moveIncriment  = moveIncriment -.25
					moveIncrimentText = str(moveIncriment)+"mm"
			elif(evalCurrentModItem == "moveXText"):
				#should check that grbl is connected!! TODO
				streamGrbl(gcodeJogX+"-"+str(moveIncriment))
			elif(evalCurrentModItem == "moveYText"):
				streamGrbl(gcodeJogY+"-"+str(moveIncriment))
			elif(evalCurrentModItem == "moveZText"):
				streamGrbl(gcodeJogZ+"-"+str(moveIncriment))
			
			
		elif(button == "right"):
			#print button
			
			#check if variable is modifiable and modify appropriately
			if(evalCurrentModItem == "manualMotorValueText"):
				if(manualMotorValue < 100):
					manualMotorValue  = manualMotorValue + 1
					manualMotorValueText = str(manualMotorValue)+"%"
			elif(evalCurrentModItem == "enableMotorText"):
				enableMotor = True
				enableMotorText = str(enableMotor)
				
				#if motor thread is not already running start it
				if (not runMotorThread.isAlive()):
					runMotorThread.start()
					
			elif(evalCurrentModItem == "enablePwmText"):
				if(not enablePwm):
					if(openSerialPwmPort()):
						enablePwm = True
						enablePwmText = str(enablePwm)
						
						#if motor thread is not already running start it
						if (not runMotorThread.isAlive()):
							runMotorThread.start()
					
			elif(evalCurrentModItem == "enableGrblText"):
				if(not enableGrbl):
					if(openSerialGrblPort()):
						enableGrbl = True
						enableGrblText = str(enableGrbl)
						#Unlock GRBL
						streamGrbl(gcodeUnlock)
			elif(evalCurrentModItem == "moveIncrimentText"):
				if(moveIncriment < 10.0):
					moveIncriment  = moveIncriment +.25
					moveIncrimentText = str(moveIncriment)+"mm"
			elif(evalCurrentModItem == "moveXText"):
				#should check that grbl is connected!! TODO
				streamGrbl(gcodeJogX+str(moveIncriment))
			elif(evalCurrentModItem == "moveYText"):
				streamGrbl(gcodeJogY+str(moveIncriment))
			elif(evalCurrentModItem == "moveZText"):
				streamGrbl(gcodeJogZ+str(moveIncriment))
		
		
	#Poll rpm reading
	rpmText = "RPM "+str(int(tach.RPM())) + " "

	#Poll PWM value
	if(enablePwm):
		serialPwmPort.flushInput()     #flush old data
		serialPwmPort.readline()       # read what may be a partion string
		
		#grblPwmValueText = serialPwmPort.readline() + " "# take in full reading
		#grblPwmValue = serialPwmPort.readline() # take in full reading
		grblPwmValueTextTmp = serialPwmPort.readline()
		grblPwmValueTmp     = 0
		
		#try to convert serial input to an intiger value
		trySuccess = False
		try:
			grblPwmValueTmp = int(grblPwmValueTextTmp)
			trySuccess = True
		except:
			print "Bad PWM value: %s" % grblPwmValueTextTmp
		
		#only save new value if its outside of jitter value
		if(trySuccess):
			if(grblPwmValueTmp > (grblPwmValue+pwmJitterReduction) or grblPwmValueTmp < (grblPwmValue-pwmJitterReduction)):
				grblPwmValue = grblPwmValueTmp
				grblPwmValueText = str(grblPwmValue)

	
	if(topOledhighlight):
		#get current values
		topOledText = eval(currentMenu)[menuLocation][0]
		bottomOledText = eval(currentMenu)[menuLocation+1][0]
		
		#check if text has $ character		
		if('$' in topOledText):
			#display variable referenced not literal text
			topOledText = eval(topOledText[1:])
			#print (topOledText)
			
		if('$' in bottomOledText):
			bottomOledText = eval(bottomOledText[1:])
			#print (bottomOledText)
	else:
		#get current values
		topOledText = eval(currentMenu)[menuLocation-1][0]
		bottomOledText = eval(currentMenu)[menuLocation][0]
		
		#check if text has $ character		
		if('$' in topOledText):
			#display variable referenced not literal text
			topOledText = eval(topOledText[1:])
			#print (topOledText)
			
		if('$' in bottomOledText):
			bottomOledText = eval(bottomOledText[1:])
			#print (bottomOledText)
	
	#check if we need to refresh OLED	
	if(topOledText != topOledTextPrev or bottomOledText != bottomOledTextPrev or topOledhighlight != topOledhighlightPrev):
		#refresh screen
		print("--refreshing screen--")
		print(topOledText)
		print(topOledTextPrev)
		print(bottomOledText)
		print(bottomOledTextPrev)
		print("---------------------")
		printScreen(topOledhighlight, topOledText, bottomOledText)
	#printScreen(topOledhighlight, topOledText, bottomOledText)	
		
	#set the last button pressed to be the current one
	buttonPrev = button
	topOledTextPrev = topOledText
	bottomOledTextPrev = bottomOledText
	topOledhighlightPrev = topOledhighlight
	
	
	#if your holding left or right button reset prevously pressed button every "buttonPressedMaxCount"
	if(button == "left" or  button == "right"):
		buttonPressedCount =  buttonPressedCount + 1 #incriment loop count while button is being pressed
		
		#if button is held long enough reset previously pressed button
		if(buttonPressedCount >= buttonPressedMaxCount): 
			buttonPrev = "NA"	
	#clear count when button is released/not pressed	
	else: 
		buttonPressedCount = 0
		
	#reduce CPU usage
	sleep(.001)
		
	

	


